@@ -2,11 +2,11 @@
  * Support rsync daemon authentication.
  *
  * Copyright (C) 1998-2000 Andrew Tridgell
- * Copyright (C) 2002, 2004, 2005, 2006 Wayne Davison
+ * Copyright (C) 2002-2008 Wayne Davison
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street - Fifth Floor, Boston, MA 02110-1301, USA.
+ * with this program; if not, visit the http://fsf.org website.
  */
 
 #include "rsync.h"
@@ -27,11 +26,11 @@ extern char *password_file;
 encode a buffer using base64 - simple and slow algorithm. null terminates
 the result.
   ***************************************************************************/
-void base64_encode(char *buf, int len, char *out, int pad)
+void base64_encode(const char *buf, int len, char *out, int pad)
 {
 	char *b64 = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
 	int bit_offset, byte_offset, idx, i;
-	unsigned char *d = (unsigned char *)buf;
+	const uchar *d = (const uchar *)buf;
 	int bytes = (len*8 + 5)/6;
 
 	for (i = 0; i < bytes; i++) {
@@ -55,15 +54,16 @@ void base64_encode(char *buf, int len, char *out, int pad)
 }
 
 /* Generate a challenge buffer and return it base64-encoded. */
-static void gen_challenge(char *addr, char *challenge)
+static void gen_challenge(const char *addr, char *challenge)
 {
 	char input[32];
-	char md4_out[MD4_SUM_LENGTH];
+	char digest[MAX_DIGEST_LEN];
 	struct timeval tv;
+	int len;
 
 	memset(input, 0, sizeof input);
 
-	strlcpy((char *)input, addr, 17);
+	strlcpy(input, addr, 17);
 	sys_gettimeofday(&tv);
 	SIVAL(input, 16, tv.tv_sec);
 	SIVAL(input, 20, tv.tv_usec);
@@ -71,20 +71,21 @@ static void gen_challenge(char *addr, char *challenge)
 
 	sum_init(0);
 	sum_update(input, sizeof input);
-	sum_end(md4_out);
+	len = sum_end(digest);
 
-	base64_encode(md4_out, MD4_SUM_LENGTH, challenge, 0);
+	base64_encode(digest, len, challenge, 0);
 }
 
 
 /* Return the secret for a user from the secret file, null terminated.
  * Maximum length is len (not counting the null). */
-static int get_secret(int module, char *user, char *secret, int len)
+static int get_secret(int module, const char *user, char *secret, int len)
 {
-	char *fname = lp_secrets_file(module);
+	const char *fname = lp_secrets_file(module);
 	STRUCT_STAT st;
 	int fd, ok = 1;
-	char ch, *p;
+	const char *p;
+	char ch, *s;
 
 	if (!fname || !*fname)
 		return 0;
@@ -136,60 +137,57 @@ static int get_secret(int module, char *user, char *secret, int len)
 	}
 
 	/* Slurp the secret into the "secret" buffer. */
-	p = secret;
+	s = secret;
 	while (len > 0) {
-		if (read(fd, p, 1) != 1 || *p == '\n')
+		if (read(fd, s, 1) != 1 || *s == '\n')
 			break;
-		if (*p == '\r')
+		if (*s == '\r')
 			continue;
-		p++;
+		s++;
 		len--;
 	}
-	*p = '\0';
+	*s = '\0';
 	close(fd);
 
 	return 1;
 }
 
-static char *getpassf(char *filename)
+static const char *getpassf(const char *filename)
 {
 	STRUCT_STAT st;
 	char buffer[512], *p;
 	int fd, n, ok = 1;
-	char *envpw = getenv("RSYNC_PASSWORD");
+	const char *envpw = getenv("RSYNC_PASSWORD");
 
 	if (!filename)
 		return NULL;
 
 	if ((fd = open(filename,O_RDONLY)) < 0) {
-		rsyserr(FERROR, errno, "could not open password file \"%s\"",
+		rsyserr(FWARNING, errno, "could not open password file \"%s\"",
 			filename);
 		if (envpw)
-			rprintf(FERROR, "falling back to RSYNC_PASSWORD environment variable.\n");
+			rprintf(FINFO, "falling back to RSYNC_PASSWORD environment variable.\n");
 		return NULL;
 	}
 
 	if (do_stat(filename, &st) == -1) {
-		rsyserr(FERROR, errno, "stat(%s)", filename);
+		rsyserr(FWARNING, errno, "stat(%s)", filename);
 		ok = 0;
 	} else if ((st.st_mode & 06) != 0) {
-		rprintf(FERROR,"password file must not be other-accessible\n");
+		rprintf(FWARNING, "password file must not be other-accessible\n");
 		ok = 0;
 	} else if (MY_UID() == 0 && st.st_uid != 0) {
-		rprintf(FERROR,"password file must be owned by root when running as root\n");
+		rprintf(FWARNING, "password file must be owned by root when running as root\n");
 		ok = 0;
 	}
 	if (!ok) {
-		rprintf(FERROR,"continuing without password file\n");
-		if (envpw)
-			rprintf(FERROR, "using RSYNC_PASSWORD environment variable.\n");
 		close(fd);
+		rprintf(FWARNING, "continuing without password file\n");
+		if (envpw)
+			rprintf(FINFO, "falling back to RSYNC_PASSWORD environment variable.\n");
 		return NULL;
 	}
 
-	if (envpw)
-		rprintf(FERROR, "RSYNC_PASSWORD environment variable ignored\n");
-
 	n = read(fd, buffer, sizeof buffer - 1);
 	close(fd);
 	if (n > 0) {
@@ -203,16 +201,17 @@ static char *getpassf(char *filename)
 
 /* Generate an MD4 hash created from the combination of the password
  * and the challenge string and return it base64-encoded. */
-static void generate_hash(char *in, char *challenge, char *out)
+static void generate_hash(const char *in, const char *challenge, char *out)
 {
-	char buf[MD4_SUM_LENGTH];
+	char buf[MAX_DIGEST_LEN];
+	int len;
 
 	sum_init(0);
 	sum_update(in, strlen(in));
 	sum_update(challenge, strlen(challenge));
-	sum_end(buf);
+	len = sum_end(buf);
 
-	base64_encode(buf, MD4_SUM_LENGTH, out, 0);
+	base64_encode(buf, len, out, 0);
 }
 
 /* Possibly negotiate authentication with the client.  Use "leader" to
@@ -221,14 +220,14 @@ static void generate_hash(char *in, char *challenge, char *out)
  * Return NULL if authentication failed.  Return "" if anonymous access.
  * Otherwise return username.
  */
-char *auth_server(int f_in, int f_out, int module, char *host, char *addr,
-		  char *leader)
+char *auth_server(int f_in, int f_out, int module, const char *host,
+		  const char *addr, const char *leader)
 {
 	char *users = lp_auth_users(module);
-	char challenge[MD4_SUM_LENGTH*2];
+	char challenge[MAX_DIGEST_LEN*2];
 	char line[BIGPATHBUFLEN];
 	char secret[512];
-	char pass2[MD4_SUM_LENGTH*2];
+	char pass2[MAX_DIGEST_LEN*2];
 	char *tok, *pass;
 
 	/* if no auth list then allow anyone in! */
@@ -239,7 +238,7 @@ char *auth_server(int f_in, int f_out, int module, char *host, char *addr,
 
 	io_printf(f_out, "%s%s\n", leader, challenge);
 
-	if (!read_line(f_in, line, sizeof line - 1)
+	if (!read_line_old(f_in, line, sizeof line)
 	 || (pass = strchr(line, ' ')) == NULL) {
 		rprintf(FLOG, "auth failed on module %s from %s (%s): "
 			"invalid challenge response\n",
@@ -286,11 +285,10 @@ char *auth_server(int f_in, int f_out, int module, char *host, char *addr,
 	return strdup(line);
 }
 
-
-void auth_client(int fd, char *user, char *challenge)
+void auth_client(int fd, const char *user, const char *challenge)
 {
-	char *pass;
-	char pass2[MD4_SUM_LENGTH*2];
+	const char *pass;
+	char pass2[MAX_DIGEST_LEN*2];
 
 	if (!user || !*user)
 		user = "nobody";
