@@ -4,11 +4,11 @@
  * Copyright (C) 1996-2000 Andrew Tridgell
  * Copyright (C) 1996 Paul Mackerras
  * Copyright (C) 2002 Martin Pool
- * Copyright (C) 2003, 2004, 2005, 2006 Wayne Davison
+ * Copyright (C) 2003-2008 Wayne Davison
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -17,8 +17,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street - Fifth Floor, Boston, MA 02110-1301, USA.
+ * with this program; if not, visit the http://fsf.org website.
  */
 
 #include "rsync.h"
@@ -27,7 +26,7 @@ extern int am_server;
 extern int am_daemon;
 extern int io_error;
 extern int keep_partial;
-extern int log_got_error;
+extern int got_xfer_error;
 extern char *partial_dir;
 extern char *logfile_name;
 
@@ -79,8 +78,8 @@ void close_all(void)
  **/
 int cleanup_got_literal = 0;
 
-static char *cleanup_fname;
-static char *cleanup_new_fname;
+static const char *cleanup_fname;
+static const char *cleanup_new_fname;
 static struct file_struct *cleanup_file;
 static int cleanup_fd_r, cleanup_fd_w;
 static pid_t cleanup_pid = 0;
@@ -104,6 +103,11 @@ NORETURN void _exit_cleanup(int code, const char *file, int line)
 	if (exit_code) /* Preserve first error code when recursing. */
 		code = exit_code;
 
+	/* If this is the exit at the end of the run, the server side
+	 * should not attempt to output a message (see log.c). */
+	if (am_server && code == 0)
+		am_server = 2;
+
 	/* Some of our actions might cause a recursive call back here, so we
 	 * keep track of where we are in the cleanup and never repeat a step. */
 	switch (cleanup_step) {
@@ -135,7 +139,7 @@ NORETURN void _exit_cleanup(int code, const char *file, int line)
 
 		if (cleanup_got_literal && cleanup_fname && cleanup_new_fname
 		 && keep_partial && handle_partial_dir(cleanup_new_fname, PDIR_CREATE)) {
-			char *fname = cleanup_fname;
+			const char *fname = cleanup_fname;
 			cleanup_fname = NULL;
 			if (cleanup_fd_r != -1)
 				close(cleanup_fd_r);
@@ -143,7 +147,7 @@ NORETURN void _exit_cleanup(int code, const char *file, int line)
 				flush_write_file(cleanup_fd_w);
 				close(cleanup_fd_w);
 			}
-			finish_transfer(cleanup_new_fname, fname, NULL,
+			finish_transfer(cleanup_new_fname, fname, NULL, NULL,
 					cleanup_file, 0, !partial_dir);
 		}
 
@@ -170,7 +174,7 @@ NORETURN void _exit_cleanup(int code, const char *file, int line)
 				code = exit_code = RERR_DEL_LIMIT;
 			if (io_error & IOERR_VANISHED)
 				code = exit_code = RERR_VANISHED;
-			if (io_error & IOERR_GENERAL || log_got_error)
+			if (io_error & IOERR_GENERAL || got_xfer_error)
 				code = exit_code = RERR_PARTIAL;
 		}
 
@@ -190,6 +194,8 @@ NORETURN void _exit_cleanup(int code, const char *file, int line)
 		/* FALLTHROUGH */
 #include "case_N.h"
 
+		if (am_server && code)
+			msleep(100);
 		close_all();
 
 		/* FALLTHROUGH */
@@ -207,7 +213,7 @@ void cleanup_disable(void)
 }
 
 
-void cleanup_set(char *fnametmp, char *fname, struct file_struct *file,
+void cleanup_set(const char *fnametmp, const char *fname, struct file_struct *file,
 		 int fd_r, int fd_w)
 {
 	cleanup_fname = fnametmp;
