@@ -26,6 +26,7 @@ int module_id = -1;
 int relative_paths = 0;
 int human_readable = 0;
 int module_dirlen = 0;
+int preserve_xattrs = 0;
 mode_t orig_umask = 002;
 char *partial_dir;
 char *module_dir;
@@ -69,6 +70,11 @@ struct filter_list_struct daemon_filter_list;
 	return -1;
 }
 
+ int copy_xattrs(UNUSED(const char *source), UNUSED(const char *dest))
+{
+	return -1;
+}
+
  char *lp_name(UNUSED(int mod))
 {
 	return NULL;
